' StuffPlug-NG 2 Language converter
' =================================
'
' Converts LNG files to managable INI files and vice-verca.
'
' The INI files are coded with UTF-8. The reason for this is to allow
' this script to convert them back to proper StuffPlug-NG language files
' without adding more code to this script (to convert everything to UTF-8)
' And because I'm lazy :P
'
' The INI files can easly be edited with notepad or even with Choli's
' translator program(*). Even if the language you're creating doesn't
' need Unicode or UTF-8 coding, you still need to save the INI file as
' UTF-8 before converting it to a StuffPlug-NG language file.
'
' Converting between UTF-8, Unicode and ASCII can be done with notepad.
'
'
' Note: This script doesn't use much error checking, so make sure
'       everything is A-OK with the files before you use the script.
'       (eg: make sure no files are locked or in use) Also there isn't
'       checking for already existing files, so they files will be
'       overwritten if they already exist.
'
'
' (*) To use Choli's translator program you first need to convert the
'     INI file to Unicode with notepad. You'll also need to convert the
'     English StuffPlug-NG language file to an INI file of course...
'
'
' Have fun
' CookieRevised


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Global stuff
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Option Explicit

Dim FSO, WSH

Const cTitle = "StuffPlug-NG LNG/INI converter"
Set FSO = CreateObject("Scripting.FileSystemObject")
Set WSH = CreateObject("WScript.Shell")


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Main routine
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim oArgs
Set oArgs = WScript.Arguments
If oArgs.Count = 0 Then
  ShowMsgBox "usage", ""
Else
  Select Case LCase(FSO.GetExtensionName(oArgs(0)))
    Case "ini": ConvertINI FSO.GetAbsolutePathName(oArgs(0))
    Case "lng": ConvertLNG FSO.GetAbsolutePathName(oArgs(0))
    Case Else:  ShowMsgBox "extension", ""
  End Select
End If
Set oArgs = Nothing
Set WSH = Nothing
Set FSO = Nothing
WScript.Quit


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Convert from UTF-8 INI to StuffPlug LNG                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub ConvertINI(sFilespec)
  If Not (FSO.FileExists(sFilespec)) Then
    ShowMsgBox "filenotfound", sFilespec
  Else
    'File must be at least 5 bytes to be valid in theory
    ' => 3 bytes for the BOM + 2 bytes minimum as first line ("k=")
    If FSO.GetFile(sFilespec).Size < 5 Then
      ShowMsgBox "invalidfile", ""
    Else
      Dim oStreamIn
      Dim oStreamOut
      Dim sBytes
      Set oStreamIn = FSO.OpenTextFile(sFilespec, 1, 0, 0)
      'Check BOM/structure
      sBytes = oStreamIn.Read(3)
      If sBytes <> Chr(&hEF) & Chr(&hBB) & Chr(&hBF) Then
        ShowMsgBox "wrongbom", ""
      Else
        Set oStreamOut = FSO.CreateTextFile(FSO.BuildPath(FSO.GetParentFolderName(sFilespec), FSO.GetBaseName(sFilespec) & ".lng"), -1, 0)
        Do While Not oStreamIn.AtEndOfStream
          sBytes = Trim(oStreamIn.ReadLine)
          'Ignore topics, comments, blanks, lines without "=", lines without keys
          If (InStr(sBytes, "=") > 1) And Not InStr("[;", Left(sBytes, 1)) Then
            oStreamOut.Write RTrim(Left(sBytes, InStr(sBytes, "=") - 1)) & Chr(0) & LTrim(Mid(sBytes, InStr(sBytes, "=") + 1)) & Chr(0)
          End If
        Loop
        oStreamOut.Close
        Set oStreamOut = Nothing
        ShowMsgBox "done", ""
      End If
      oStreamIn.Close
      Set oStreamIn = Nothing
    End If
  End If
End Sub


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Convert from StuffPlug LNG to UTF-8 INI                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub ConvertLNG(sFilespec)
  If Not (FSO.FileExists(sFilespec)) Then
    ShowMsgBox "filenotfound", sFilespec
  Else
    'File must be at least 3 bytes to be valid in theory
    ' => zero-terminated 1 character key string + zero-terminated empty value string
    If FSO.GetFile(sFilespec).Size < 3 Then
      ShowMsgBox "invalidfile", ""
    Else
      Dim oStreamIn
      Dim oStreamOut
      Dim sBytes, aBytes
      Dim I
      Set oStreamIn = FSO.OpenTextFile(sFilespec, 1, 0, 0)
      'Normaly this line should be "oStreamIn.ReadAll". But for some dark reason this doesn't work
      sBytes = oStreamIn.Read(FSO.GetFile(sFilespec).Size)
      oStreamIn.Close
      Set oStreamIn = Nothing
      'Check BOM/structure
      If (Right(sBytes, 1) <> Chr(0)) And (InStr(Mid(sBytes, 2, Len(sBytes) - 2), Chr(0)) = 0) Then
        ShowMsgBox "invalidfile", ""
      Else
        aBytes = Split(sBytes, Chr(0))
        Set oStreamOut = FSO.CreateTextFile(FSO.BuildPath(FSO.GetParentFolderName(sFilespec), FSO.GetBaseName(sFilespec) & ".ini"), -1, 0)
        oStreamOut.WriteLine Chr(&hEF) & Chr(&hBB) & Chr(&hBF) & "; This file is created with " & UCase(WScript.ScriptName) & vbCrLf & "; This file should be saved as UTF8 before converting back." & vbCrLf & ";" & vbCrLf & "; Tip: Save it as UNICODE to use it with Choli's translator." & vbCrLf & ";      When finished, don't forget to save it again as UTF8." & vbCrLf & vbCrLf & "[StuffPlug-NG]"
        For I = 0 To UBound(aBytes) - 1 Step 2
          oStreamOut.WriteLine aBytes(I) & "=" & aBytes(I + 1)
        Next
        oStreamOut.Close
        Set oStreamOut = Nothing
        ShowMsgBox "done", ""
      End If
    End If
  End If
End Sub


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Some replies you can get                                           '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub ShowMsgBox(sBoxType, sArg)
  Select Case sBoxType
    Case "usage"
      MsgBox _
        "SYNTAX: " & UCase(WScript.ScriptName) & " <LANGUAGEFILE>|<INIFILE>" & vbCrLf & vbCrLf & vbCrLf & _
        "Example:" & vbCrLf & _
        "    " & UCase(WScript.ScriptName) & " english.lng" & vbCrLf & _
        "    -> This will convert english.lng to a readable ini-file named english.ini." & vbCrLf & vbCrLf & _
        "Example:" & vbCrLf & _
        "    " & UCase(WScript.ScriptName) & " english.ini" & vbCrLf & _
        "    -> This will convert english.ini to a proper languagefile named english.lng." & vbCrLf & vbCrLf & _
        "Usage:" & vbCrLf & _
        "    In Windows Explorer drag and drop a languagefile or an ini-file on to this script." & vbCrLf & _
        "    Or use ""Start > Run"" and the commandline syntax described above." & vbCrLf & _
        "    Long filenames and paths should be enclosed in quotes as useual.", _
        vbOKOnly Or vbInformation, cTitle & " - INFO"
    Case "extension"
      MsgBox _
        "The file needs to have the correct extension so it can be reconized." & vbCrLf & _
        "Either use a languagefile (with LNG extension) or an ini-file (with INI extension)." & vbCrLf & vbCrLf & _
        "(for the correct commandline syntax run this script without any arguments)", _
        vbOKOnly Or vbCritical, cTitle & " - ERROR"
    Case "filenotfound"
      MsgBox _
        "File not found:" & vbCrLf & _
        """" & sArg & """" & vbCrLf & vbCrLf & _
        "(for the correct commandline syntax run this script without any arguments)", _
        vbOKOnly Or vbCritical, cTitle & " - ERROR"
    Case "invalidfile"
      MsgBox _
        "File is invalid:" & vbCrLf & _
        """" & sArg & """" & vbCrLf & vbCrLf & _
        "(for the correct commandline syntax run this script without any arguments)", _
        vbOKOnly Or vbCritical, cTitle & " - ERROR"
    Case "wrongbom"
      MsgBox _
        "The ini-file is not saved in UTF8 format." & vbCrLf & _
        UCase(WScript.ScriptName) & " can only handle UTF8 ini-files." & vbCrLf & _
        "You can use notepad to open, edit, and save ini-files in UTF8 format." & vbCrLf & vbCrLf & _
        "(for the correct commandline syntax run this script without any arguments)", _
        vbOKOnly Or vbCritical, cTitle & " - ERROR"
    Case "done"
      MsgBox _
        "File succesfully converted.", _
        vbOKOnly Or vbInformation, cTitle & " - INFO"
  End Select
End Sub
